package com.dsic.gj_erp.controller.zyjh;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dsic.gj_erp.bean.ResultBean;
import com.dsic.gj_erp.bean.jhgk.DmYdjh;
import com.dsic.gj_erp.bean.zyjh.DmYlInfo;
import com.dsic.gj_erp.controller.zyjh.dto.YlSlDto;
import com.dsic.gj_erp.exception.ServiceException;
import com.dsic.gj_erp.service.jhgk.DmYdjhService;
import com.dsic.gj_erp.service.zyjh.DmYlInfoService;
import lombok.AllArgsConstructor;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 余料管理
 */
@RestController
@RequestMapping("yl")
@AllArgsConstructor
public class YlController {

    private final DmYlInfoService ylInfoService;
    private final DmYdjhService ydjhService;

    @PostMapping("list")
    public ResultBean<?> list(@RequestBody YlSlDto dto){
        return new ResultBean<>(
                ylInfoService.list(Wrappers.<DmYlInfo>lambdaQuery()
                        .eq(DmYlInfo::get来源船号,dto.getDcCh())
                        .eq(DmYlInfo::get使用状态,dto.getStatus())
                        //查询结果使用最小匹配原则为按规格匹配上料提供匹配顺序
                        .orderByAsc(DmYlInfo::get板宽,DmYlInfo::get板长)
                )
        );
    }

    /**
     * 保存功能
     */
    @PostMapping("edit")
    @Transactional(rollbackFor = Exception.class)
    public ResultBean<?> edit(@RequestBody List<DmYlInfo> list){
        ylInfoService.updateBatchById(list);
        return new ResultBean<>();
    }

    /**
     * 退回
     */
    @PostMapping("rollback")
    @Transactional(rollbackFor = Exception.class)
    public ResultBean<?> rollback(@RequestBody List<DmYlInfo> list){
        List<DmYdjh> updateList=new ArrayList<>();
        list.stream()
        .filter(item->item.get使用状态()<=2)
        .forEach(item->{
            DmYdjh one = ydjhService.getOne(Wrappers.<DmYdjh>lambdaQuery()
                    .eq(DmYdjh::getDcCh, item.get使用船号())
                    .eq(DmYdjh::getTzbh, item.get使用图纸())
            );
            //退回作业计划
            //fixme 修正作业计划,余料上料相关属性
            if (ObjectUtil.isNotEmpty(one)){

            }
            //退回余料库存信息
            item.修正退回信息();
        });
        ydjhService.updateBatchById(updateList);
        this.edit(list);
        return new ResultBean<>();
    }

    /**
     * 打印
     */
    @PostMapping("print")
    @Transactional(rollbackFor = Exception.class)
    public ResultBean<?> print(@RequestBody List<DmYlInfo> list){
        ArrayList<DmYdjh> updateList = new ArrayList<>();
        list.forEach(item->{
            if (StrUtil.isAllNotEmpty(item.get使用船号(),item.get使用船号())){
                String date = DateUtil.date().toString("yyyy/MM/dd");
                item.set使用日期(date);
                item.set使用状态(2);//已打印
                List<DmYdjh> ydjhList = ydjhService.list(Wrappers.<DmYdjh>lambdaQuery()
                        .eq(DmYdjh::getDcCh, item.get使用船号())
                        .eq(DmYdjh::getTzbh,item.get使用图纸())
                );
                //防止手动编辑错误时的校验
                if (ObjectUtil.isEmpty(ydjhList)){
                    String str=StrUtil.format("使用船号:{},使用图号{}错误,请核对修正后再打印", item.get使用船号(), item.get使用图纸());
                    throw new ServiceException(99999,str);
                }
                List<DmYdjh> collect = ydjhList.stream().map(it -> {
                    //防止手动编辑错误时的校验
                    if (it.图纸已使用()){
                        String str=StrUtil.format("作业计划船号:{},图号{}已占用,请核对修正后再打印", item.get使用船号(), item.get使用图纸());
                        throw new ServiceException(99999,str);
                    }

                    return it.ofPrintForSl(item,date);
                }).collect(Collectors.toList());

                updateList.addAll(collect);
            }
        });

        ydjhService.updateBatchById(updateList);
        ylInfoService.updateBatchById(list);

        return new ResultBean<>();
    }

    /**
     * 根据余料库存数据选择结果匹配作业计划
     * 匹配成功后返回匹配结果并显示到
     */
    @PostMapping("sl")
    public ResultBean<?> sl(@RequestBody YlSlDto dto){
        if (!dto.isAllow()){
            throw new ServiceException(99999,"请填写需求船号和批次");
        }
        if (ObjectUtil.isEmpty(dto.getYlList())){
            return new ResultBean<>(new ArrayList<>());
        }

        List<DmYdjh> list = ydjhService.list(Wrappers.<DmYdjh>lambdaQuery()
                .eq(DmYdjh::getDcCh, dto.getDcCh())
                .eq(DmYdjh::getDcPl, dto.getDcPl())
                .eq(DmYdjh::getYlbs,"Y")
                .and(item->item.isNull(DmYdjh::getYlCh).or().eq(DmYdjh::getYlCh,""))
                .eq(StrUtil.isNotEmpty(dto.getTzbh()),DmYdjh::getTzbh,dto.getTzbh())
                .orderByDesc(DmYdjh::getWpgg)
        );

        List<DmYlInfo> 匹配成功的余料信息=new ArrayList<>();

        out:
        for (DmYdjh ydjh:list){
            for(DmYlInfo ylInfo:dto.getYlList()){
                if (ylInfo.指定余料编号的上料匹配(ydjh.getYlbhgg())){//暂时不支持,仅保留分支流程
                    ylInfo.修正上料信息(ydjh);
                    匹配成功的余料信息.add(ylInfo);
                    continue out;
                }
            }
            for (DmYlInfo ylInfo:dto.getYlList()){
                if (ylInfo.上料匹配(ydjh)){
                    ylInfo.修正上料信息(ydjh);
                    匹配成功的余料信息.add(ylInfo);
                    continue out;
                }
            }
        }

        return new ResultBean<>(匹配成功的余料信息);
    }
}
